<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'splitbrain\\slika\\tests\\' => array($vendorDir . '/splitbrain/slika/tests'),
    'splitbrain\\slika\\' => array($vendorDir . '/splitbrain/slika/src'),
    'splitbrain\\phpcli\\' => array($vendorDir . '/splitbrain/php-cli/src'),
    'splitbrain\\PHPArchive\\' => array($vendorDir . '/splitbrain/php-archive/src'),
    'phpseclib\\' => array($vendorDir . '/phpseclib/phpseclib/phpseclib'),
    'IXR\\' => array($vendorDir . '/kissifrot/php-ixr/src'),
);
