<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author qezwan <qezwan@gmail.com>
 */
$lang['connectfail']           = 'LDAP ناتوانێت پەیوەندی بکات: %s';
$lang['domainfail']            = 'LDAP ناتوانێت dn ی بەکارهێنەرەکەت بدۆزێتەوە';
