<?php return array(
    'root' => array(
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'reference' => '11ad7f4705aea641b0f6f4997c153001bc4522d6',
        'name' => 'splitbrain/dokuwiki',
        'dev' => true,
    ),
    'versions' => array(
        'aziraphale/email-address-validator' => array(
            'pretty_version' => '2.0.1',
            'version' => '2.0.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../aziraphale/email-address-validator',
            'aliases' => array(),
            'reference' => 'fa25bc22c1c0b6491657c91473fae3e40719a650',
            'dev_requirement' => false,
        ),
        'geshi/geshi' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'type' => 'library',
            'install_path' => __DIR__ . '/../geshi/geshi',
            'aliases' => array(
                0 => '1.0.x-dev',
                1 => '9999999-dev',
            ),
            'reference' => '3c12a7931d509c5e3557c5ed44c9a32e9c917c7d',
            'dev_requirement' => false,
        ),
        'kissifrot/php-ixr' => array(
            'pretty_version' => '1.8.3',
            'version' => '1.8.3.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../kissifrot/php-ixr',
            'aliases' => array(),
            'reference' => '4477cd1a67416ce5b6a2080f9a79d9eb50a965c1',
            'dev_requirement' => false,
        ),
        'marcusschwarz/lesserphp' => array(
            'pretty_version' => 'v0.5.5',
            'version' => '0.5.5.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../marcusschwarz/lesserphp',
            'aliases' => array(),
            'reference' => '77ba82b5218ff228267d3b0e5ec8697be75e86a7',
            'dev_requirement' => false,
        ),
        'openpsa/universalfeedcreator' => array(
            'pretty_version' => 'v1.8.4.1',
            'version' => '1.8.4.1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../openpsa/universalfeedcreator',
            'aliases' => array(),
            'reference' => 'e4736a68eef454a83acd100230a2e15b424f899f',
            'dev_requirement' => false,
        ),
        'phpseclib/phpseclib' => array(
            'pretty_version' => '2.0.31',
            'version' => '2.0.31.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpseclib/phpseclib',
            'aliases' => array(),
            'reference' => '233a920cb38636a43b18d428f9a8db1f0a1a08f4',
            'dev_requirement' => false,
        ),
        'simplepie/simplepie' => array(
            'pretty_version' => '1.5.6',
            'version' => '1.5.6.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../simplepie/simplepie',
            'aliases' => array(),
            'reference' => '1c68e14ca3ac84346b6e6fe3c5eedf725d0f92c6',
            'dev_requirement' => false,
        ),
        'splitbrain/dokuwiki' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'reference' => '11ad7f4705aea641b0f6f4997c153001bc4522d6',
            'dev_requirement' => false,
        ),
        'splitbrain/php-archive' => array(
            'pretty_version' => '1.2.1',
            'version' => '1.2.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../splitbrain/php-archive',
            'aliases' => array(),
            'reference' => '211a2198b73b233d7d2b6159462e11cd9a91348a',
            'dev_requirement' => false,
        ),
        'splitbrain/php-cli' => array(
            'pretty_version' => '1.1.8',
            'version' => '1.1.8.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../splitbrain/php-cli',
            'aliases' => array(),
            'reference' => '8c2c001b1b55d194402cf18aad2757049ac6d575',
            'dev_requirement' => false,
        ),
        'splitbrain/slika' => array(
            'pretty_version' => '1.0.5',
            'version' => '1.0.5.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../splitbrain/slika',
            'aliases' => array(),
            'reference' => 'be0785cb6b7def847df5d92e0e5fde57def7220f',
            'dev_requirement' => false,
        ),
    ),
);
